/*
 * This file is part of MyAdFilter <https://www.myadfilter.com/>,
 * Copyright (C) 2015-2016 DYP Solutions
 *
 * MyAdFilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * MyAdFilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
 */

let {LimitedSiteFilter, BlacklistDomainFilter} = require("filterClasses");

function DomainFilterList()
{
    this.clear();
}
DomainFilterList.prototype = {
  /**
   * list of css selector for the domain
   * domain -> [ selector -> parentSelector, ... ]
   */
  domainSelectors: null,

  /**
   * Removes all known filters
   */
  clear: function()
  {
    this.domainSelectors = Object.create(null);
  },

  add: function(filter)
  {
    if (!(filter instanceof LimitedSiteFilter))
      return;

    let domain = filter.domain;

    if (!(domain in this.domainSelectors))
      this.domainSelectors[ domain ] = Object.create(null);

    if (!filter.selector)
      return;

    if (!filter.hasParentSelector()) {
      this.domainSelectors[ domain ][ filter.selector ] = false;
    } else {
      if (!(filter.selector in this.domainSelectors[ domain ]))
        this.domainSelectors[ domain ][ filter.selector ] = [];
      
      if (
        // warning : have a more global filter which does not have parent selector
        this.domainSelectors[ domain ][ filter.selector ] === false
        // already have this parent selector
        || this.domainSelectors[ domain ][ filter.selector ].indexOf( filter.parentSelector ) >= 0
      ) {
        return;
      }
      this.domainSelectors[ domain ][ filter.selector ].push(filter.parentSelector);
    }
  },

  remove: function(filter)
  {
    if (!(filter instanceof LimitedSiteFilter))
      return;

    if (filter.domain in this.domainSelectors) {
      delete this.domainSelectors[ filter.domain ];
    }
  },

  getDomain: function(url)
  {
    let scheme = url.indexOf('://');
    if (scheme != -1)
      url = url.substring(scheme + 3);
    let slash = url.indexOf('/');
    if (slash != -1)
      url = url.substring(0, slash)

    return url;
  },

  hasDomain: function(domain)
  {
    if (!domain)
      return false;

    let domain = this.getDomain(domain);
    if (domain in this.domainSelectors)
      return true;
    return false;
  },

  getDomainSelectors: function(domain)
  {
    return this.domainSelectors[ this.getDomain(domain) ] || {};
  }
}


/**
 * shared limited site list
 */
let globalLimitedSiteList = exports.globalSiteList = new DomainFilterList();
let globalBlacklistList = exports.globalBlacklistList = new DomainFilterList();