/*
 * This file is part of MyAdFilter <https://www.myadfilter.com/>,
 * Copyright (C) 2006-2015 Eyeo GmbH
 * Copyright (C) 2015-2016 DYP Solutions
 *
 * MyAdFilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * MyAdFilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MyAdFilter.  If not, see <http://www.gnu.org/licenses/>.
 */

(function()
{
  var backgroundPage = ext.backgroundPage.getWindow();
  var require = backgroundPage.require;
  var getBlockedPerPage = require("stats").getBlockedPerPage;
  var FilterNotifier = require("filterNotifier").FilterNotifier;
  var Prefs = require("prefs").Prefs;
  
  var currentPage;
  
  var messageMark = {};

  function onLoad()
  {
    // document.getElementById("share-box").addEventListener("click", share, false);
    var showIconNumber = document.getElementById("show-iconnumber");
    showIconNumber.setAttribute("aria-checked", Prefs.show_statsinicon);
    showIconNumber.addEventListener("click", toggleIconNumber, false);
    document.querySelector("label[for='show-iconnumber']").addEventListener("click", toggleIconNumber, false);
    
    // Update stats
    ext.pages.query({active: true, lastFocusedWindow: true}, function(pages)
    {
      currentPage = pages[0];
      updateStats();

      FilterNotifier.addListener(onNotify);

      document.getElementById("stats-container").removeAttribute("hidden");
    });
  }
  
  function onUnload()
  {
    FilterNotifier.removeListener(onNotify);
  }
  
  function onNotify(action, item)
  {
    if (action == "filter.hitCount")
      updateStats();
  }
  
  function updateStats()
  {
    var statsPage = document.getElementById("stats-page");
    var blockedPage = getBlockedPerPage(currentPage).toLocaleString();
    i18n.setElementText(statsPage, "stats_label_page", [blockedPage]);
    
    var statsTotal = document.getElementById("stats-total");
    var blockedTotal = Prefs.blocked_total.toLocaleString();
    i18n.setElementText(statsTotal, "stats_label_total", [blockedTotal]);
  }
  
  // function share(ev)
  // {
  //   // Easter Egg
  //   var blocked = Prefs.blocked_total;
  //   if (blocked <= 9000 || blocked >= 10000)
  //     blocked = blocked.toLocaleString();
  //   else
  //     blocked = i18n.getMessage("stats_over", (9000).toLocaleString());
    
  //   ext.pages.open(createShareLink(ev.target.dataset.social, blocked));
  // }
  
  function toggleIconNumber()
  {
    Prefs.show_statsinicon = !Prefs.show_statsinicon;
    document.getElementById("show-iconnumber").setAttribute("aria-checked", Prefs.show_statsinicon);
  }
  
  document.addEventListener("DOMContentLoaded", onLoad, false);
  window.addEventListener("unload", onUnload, false);
})();
